import psycopg2
import pandas as pd

class Redshift(object):
    """
    Redshift client which connect to redshfit database.
    Furthermore, you can read sql from Redshift and
    returns the reuslt with pandas dataframe structure
    """

    def __init__(self, config=None):
        self.config = config
        self.con_pg = None
        if config is not None:
            self.con_pg = self.connect(config=config)

    def create_engine(self, dbname, user, password, host, port):
        self.config = dict(
            dbname=dbname,
            user=user,
            password=password,
            host=host,
            port=port
        )
        self.con_pg = self.connect(config=self.config)

    def connect(self, *args,**kwargs):
        config = kwargs['config']
        try:
            con_pg=psycopg2.connect(
                dbname=config['dbname'],
                host=config['host'],
                port=config['port'],
                user=config['user'],
                password=config['password']
            )
            return con_pg
        except Exception as err:
            print(err)

    def read_sql(self, sql, index_col=None, columns=None, count=0):
        return pd.read_sql(sql, self.con_pg, index_col, columns=columns)

class RedshiftConfigurationError(Exception):

    def __init__(self,
            expr="ConfigError",
            msg="Config does not exists. create engine"):
        self.expr = expr
        self.msg = msg

    def __str__(self):
        return "{} {}".format(self.expr, self.msg)


class RedshiftConnectionError(Exception):
    """Exception raised for errors in the Redshift connection.

    Attributes:
        expr -- input expression in which the error occurred
        msg  -- explanation of the error
    """

    def __init__(self, expr, msg="Failed to connect"):
        self.expr = expr
        self.msg = msg

    def __str__(self):
        return "{} {}".format(self.expr, self.msg)


def create_engine(dbname, user, password, host, port):
    rs.create_engine(dbname, user, password, host, port)

def read_sql(sql, index_col=None, columns=None):
    if rs.con_pg is None: raise RedshiftConfigurationError()
    return  rs.read_sql(sql, index_col, columns)

# HACK:FIX.
# this code is really ugly
# I did so because I wanted to store the configuration, and
# wanted users to use `read_sql` without setting cursor everytime they access to DB
rs = Redshift()



try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

import re
import os

def pandas_rs_version():
    version_regex  = re.compile(r'__version__ = "([^\"]*)"')
    return version_regex.match(
        open('pandas_rs/version.py').read()
    ).group(1)

def read_file(filename):
    filepath = os.path.join(
        os.path.dirname(
            os.path.dirname(__file__)
        ),
        filename
    )
    if os.path.exists(filepath):
        return open(filepath).read()
    else:
        return ''

setup(
    name = "pandas-rs",
    packages = ["pandas_rs"],
    install_requires = ["pandas", "psycopg2"],
    version = pandas_rs_version(),
    description = "pandas extension for AWS RedShift (Not Officail library)",
    author = "Tatsuro Yasukawa",
    author_email = "t.yasukawa01@gmail.com",
    url = "https://github.com/SamuraiT/pandas-rs",
    download_url = "",
    keywords = ["pandas", "RedShift", "sql"],
    classifiers = [
        "Programming Language :: Python :: 3",
        "Development Status :: 4 - Beta",
        "Environment :: Other Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    long_description = read_file('README.md')
)